//
//  Demo1ViewController.m
//  Scene Kit Demo 1
//
//  Created by Demitri Muna on 13/09/2012.
//  Copyright (c) 2012 Demitri Muna. All rights reserved.
//

#import "Demo1ViewController.h"
#import <SceneKit/SceneKit.h>

#define PI 3.14159265358979323846
#define deg2rad(DEG) ((DEG)*((PI)/(180.0)))

@interface Demo1ViewController ()

@end

@implementation Demo1ViewController

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Initialization code here.
    }
    
    return self;
}

- (void)awakeFromNib
{
	SCNView *sceneView = (SCNView *)self.view;
	sceneView.backgroundColor = [NSColor colorWithCalibratedWhite:0.284 alpha:1.0];
	sceneView.allowsCameraControl = YES;

	// create an empty scene and set it to the view
	SCNScene *scene = [SCNScene scene];
	sceneView.scene = scene;
	
	// Create a camera
	// ---------------
	SCNCamera *camera = [SCNCamera camera];
	
	// set field of view - can make wide or narrow
	camera.xFov = 45;   // Degrees, not radians
	camera.yFov = 45;
	
	// create a node for the camera
	SCNNode *cameraNode = [SCNNode node];
	cameraNode.camera = camera;
	cameraNode.position = SCNVector3Make(0, 0, 30);
	
	// add camera to root node
	[scene.rootNode addChildNode:cameraNode];
	
	// Create a sphere
	// ---------------
	SCNSphere *sphere = [SCNSphere sphereWithRadius:10];
	SCNNode *sphereNode = [SCNNode nodeWithGeometry:sphere];
	//CATransform3D rotation = CATransform3DMakeRotation(deg2rad(45), 1, 0, 0);
	//sphereNode.transform = rotation;
	[scene.rootNode addChildNode:sphereNode];
	
    
	SCNBox *box = [SCNBox boxWithWidth:5 height:5 length:5 chamferRadius:0.2];
	SCNNode *boxNode = [SCNNode nodeWithGeometry:box];
	boxNode.position = SCNVector3Make(10, 10, 0);
	//boxNode.transform = CATransform3DScale(boxNode.transform, 0.5, 0.5, 0.05);
	[scene.rootNode addChildNode:boxNode];
	
    
	// define the material for the sphere
	SCNMaterial *sphereMaterial = [SCNMaterial material];
	sphereMaterial.diffuse.contents = [NSColor redColor]; // colors shown when hit by diffuse light - image, or NSColor
	sphereMaterial.specular.contents = [NSColor whiteColor]; // colors shown when hit by specular light
	sphereMaterial.ambient.contents = [NSColor blueColor]; // colors shown when hit by diffuse light - image, or NSColor
	sphereMaterial.shininess = 0.25; // shape of specular highlight
	sphere.materials = @[sphereMaterial];
	
	
	// Let there be light(s)!
	// ----------------------
	
	SCNLight *ambientLight = [SCNLight light];
	SCNNode *ambientLightNode = [SCNNode node]; // note light is a node too
	ambientLight.type = SCNLightTypeAmbient;
	ambientLight.color = [NSColor colorWithDeviceWhite:0.5 alpha:1.0];
	ambientLightNode.light = ambientLight;
	[scene.rootNode addChildNode:ambientLightNode];
	

	SCNLight *specularLight = [SCNLight light];
	SCNNode *specularLightNode = [SCNNode node];
	specularLight.type = SCNLightTypeDirectional;
	specularLight.color = [NSColor greenColor];
	specularLightNode.light = specularLight;
	[scene.rootNode addChildNode:specularLightNode];


	// Create a diffuse light (omnidirectional)
	SCNLight *diffuseLight = [SCNLight light];
	SCNNode *diffuseLightNode = [SCNNode node];
	diffuseLight.type = SCNLightTypeOmni;
	diffuseLightNode.light = diffuseLight;
	diffuseLightNode.position = SCNVector3Make(-30, 30, 50);
	[scene.rootNode addChildNode:diffuseLightNode];

}

@end
