//
//  DMSceneView.m
//  SceneKitPlayground
//
//  Created by Demitri Muna on 29/08/2012.
//  Copyright (c) 2012 Demitri Muna. All rights reserved.
//

#define PI 3.14159265358979323846
#define deg2rad(DEG) ((DEG)*((PI)/(180.0)))
#define ROTATION_FACTOR 0.003

#import "DMSceneView.h"

#include <Accelerate/Accelerate.h>

@implementation DMSceneView
{
	NSPoint       _clickLocation;
	CATransform3D _initialTransform;
	
}

- (id)initWithFrame:(NSRect)frame
{
    self = [super initWithFrame:frame];
    if (self) {
        // Initialization code here.
		rotation = 0;
    }
    
    return self;
}

/*
- (void)drawRect:(NSRect)dirtyRect
{
    // Drawing code here.
}
*/
- (void)awakeFromNib
{
	self.backgroundColor = [NSColor colorWithCalibratedWhite:0.284 alpha:1.0];
	self.allowsCameraControl = YES;
	
	// Create an empty scene
	SCNScene *scene = [SCNScene scene];
	self.scene = scene;
	
	// Create a camera
	SCNCamera *camera = [SCNCamera camera];
	camera.xFov = 45;   // Degrees, not radians
	camera.yFov = 45;
	SCNNode *cameraNode = [SCNNode node];
	cameraNode.camera = camera;
	cameraNode.position = SCNVector3Make(0, 0, 30);
	[scene.rootNode addChildNode:cameraNode];
	
	/*
	// Create a torus
	SCNTorus *torus = [SCNTorus torusWithRingRadius:8 pipeRadius:1.5];
	SCNNode *torusNode = [SCNNode nodeWithGeometry:torus];
	CATransform3D torusRot = CATransform3DMakeRotation(deg2rad(45), 1, 0, 0);
	torusNode.transform = torusRot;
	[scene.rootNode addChildNode:torusNode];
	 */
	// Text
	SCNText *text = [SCNText textWithString:@"x" extrusionDepth:10.f];
	text.font = [NSFont fontWithName:@"Avenir Next Heavy" size:40.];
	text.chamferRadius = 1.f;
	self.textNode = [SCNNode nodeWithGeometry:text];
	self.textNode.position = SCNVector3Make(10, 0, 0);
	self.textNode.transform = CATransform3DScale(self.textNode.transform, .1f, .1f, .1f);
	[scene.rootNode addChildNode:self.textNode];
	
	SCNMaterial *textMaterial = [SCNMaterial material];
	textMaterial.ambient.contents = [NSColor yellowColor];
	SCNMaterial *moonMaterial = [SCNMaterial material];
	moonMaterial.ambient.contents = [NSImage imageNamed:@"moon-2048"];
	text.materials = @[moonMaterial, textMaterial];
	//	[torusNode addChildNode:self.textNode];
	
	// Create a sphere
	self.sphere = [SCNSphere sphereWithRadius:10];
	self.sphereNode = [SCNNode nodeWithGeometry:self.sphere];
	self.sphereNode.transform = CATransform3DMakeRotation(deg2rad(45), 1, 0, 0); // angles in radians
	[scene.rootNode addChildNode:self.sphereNode];
	
	self.sphereMaterial = [SCNMaterial material];
	//sphereMaterial.diffuse.contents = [NSImage imageNamed:@"eso0932a"]; // colors shown when hit by diffuse light - image, or NSColor
	//sphereMaterial.specular.contents = [NSColor whiteColor]; // colors shown when hit by specular light
	self.sphereMaterial.ambient.contents = [NSImage imageNamed:@"eso0932a"]; // colors shown when hit by ambient light - image, or NSColor
	self.sphereMaterial.shininess = 0.5; // shape of specular highlight
	self.sphere.materials = @[self.sphereMaterial];
	
	// arrow
	SCNText *textArrow = [SCNText textWithString:@"➜" extrusionDepth:3.f];
	SCNNode *textArrowNode = [SCNNode nodeWithGeometry:textArrow];
	textArrowNode.position = SCNVector3Make(10, 10*cos(0.785398163), 0);
	textArrowNode.transform = CATransform3DScale(textArrowNode.transform, .05f, .05f, .05f);
	[self.sphereNode addChildNode:textArrowNode];

	// arrow color
	SCNMaterial *arrowMaterial = [SCNMaterial material];
	arrowMaterial.ambient.contents = [NSColor blueColor];
	textArrow.materials = @[arrowMaterial];
	
	// Create ambient light
	SCNLight *ambientLight = [SCNLight light];
	SCNNode *ambientLightNode = [SCNNode node];
	ambientLight.type = SCNLightTypeAmbient;
	ambientLight.color = [NSColor colorWithDeviceWhite:1 alpha:1.0];
	ambientLightNode.light = ambientLight;
	[scene.rootNode addChildNode:ambientLightNode];
	
	/*	SCNLight *specularLight = [SCNLight light];
	 SCNNode *specularLightNode = [SCNNode node];
	 specularLight.type = SCNLightTypeDirectional;
	 specularLight.color = [NSColor greenColor];
	 specularLightNode.light = specularLight;
	 [scene.rootNode addChildNode:specularLightNode];*/
	
	// Create a diffuse light
	SCNLight *diffuseLight = [SCNLight light];
	SCNNode *diffuseLightNode = [SCNNode node];
	diffuseLight.type = SCNLightTypeOmni;
	diffuseLightNode.light = diffuseLight;
	diffuseLightNode.position = SCNVector3Make(-30, 30, 50);
	//[scene.rootNode addChildNode:diffuseLightNode];
/*
	CAKeyframeAnimation *animation = [CAKeyframeAnimation animationWithKeyPath:@"transform"];
	animation.values = [NSArray arrayWithObjects:
						[NSValue valueWithCATransform3D:CATransform3DRotate(torusNode.transform, 0 * M_PI / 2, 1.f, 0.5f, 0.f)],
						[NSValue valueWithCATransform3D:CATransform3DRotate(torusNode.transform, 1 * M_PI / 2, 1.f, 0.5f, 0.f)],
						[NSValue valueWithCATransform3D:CATransform3DRotate(torusNode.transform, 2 * M_PI / 2, 1.f, 0.5f, 0.f)],
						[NSValue valueWithCATransform3D:CATransform3DRotate(torusNode.transform, 3 * M_PI / 2, 1.f, 0.5f, 0.f)],
						[NSValue valueWithCATransform3D:CATransform3DRotate(torusNode.transform, 4 * M_PI / 2, 1.f, 0.5f, 0.f)],
						nil];
	animation.duration = 3.f;
	animation.repeatCount = HUGE_VALF;
	
	//[torusNode addAnimation:animation forKey:@"transform"];
*/
	CAKeyframeAnimation *sphereAnimation = [CAKeyframeAnimation animationWithKeyPath:@"transform"];
	sphereAnimation.values = [NSArray arrayWithObjects:
						[NSValue valueWithCATransform3D:CATransform3DRotate(self.sphereNode.transform, 0 * M_PI / 2, 1.f, 0.5f, 0.f)],
						[NSValue valueWithCATransform3D:CATransform3DRotate(self.sphereNode.transform, 1 * M_PI / 2, 1.f, 0.5f, 0.f)],
						[NSValue valueWithCATransform3D:CATransform3DRotate(self.sphereNode.transform, 2 * M_PI / 2, 1.f, 0.5f, 0.f)],
						[NSValue valueWithCATransform3D:CATransform3DRotate(self.sphereNode.transform, 3 * M_PI / 2, 1.f, 0.5f, 0.f)],
						[NSValue valueWithCATransform3D:CATransform3DRotate(self.sphereNode.transform, 4 * M_PI / 2, 1.f, 0.5f, 0.f)],
						nil];
	sphereAnimation.duration = 20.f;
	sphereAnimation.repeatCount = HUGE_VALF;

	//[self.sphereNode addAnimation:sphereAnimation forKey:@"transform"];

	// Animate the text
	CAKeyframeAnimation *textAnimation = [CAKeyframeAnimation animationWithKeyPath:@"transform"];
	textAnimation.values = [NSArray arrayWithObjects:
							[NSValue valueWithCATransform3D:CATransform3DRotate(self.textNode.transform, 0 * M_PI / 2, 0.f, 0.5f, 1.f)],
							[NSValue valueWithCATransform3D:CATransform3DRotate(self.textNode.transform, 1 * M_PI / 2, 0.f, 0.5f, 1.f)],
							[NSValue valueWithCATransform3D:CATransform3DRotate(self.textNode.transform, 2 * M_PI / 2, 0.f, 0.5f, 1.f)],
							[NSValue valueWithCATransform3D:CATransform3DRotate(self.textNode.transform, 3 * M_PI / 2, 0.f, 0.5f, 1.f)],
							[NSValue valueWithCATransform3D:CATransform3DRotate(self.textNode.transform, 4 * M_PI / 2, 0.f, 0.5f, 1.f)],
							nil];
	textAnimation.duration = 3.f;
	textAnimation.repeatCount = HUGE_VALF;
	
	
	
	//[self.textNode addAnimation:textAnimation forKey:@"transform"];

}


- (void)selectNode:(SCNNode *)node geometryElementIndex:(NSUInteger)index {
    // Unhighlight the previous selection.
    [self.selectedMaterial.emission removeAllAnimations];
    
    // Clear the selection.
    self.selectedMaterial = nil;
    
    // Highight the selection, if there is one.
    if (node != nil) {
        // Convert the geometry element index to a material index.
        index = index % [node.geometry.materials count];
        
        // Make the material unique (i.e. unshared).
        SCNMaterial *unsharedMaterial = [[node.geometry.materials objectAtIndex:index] copy];
        [node.geometry replaceMaterialAtIndex:index withMaterial:unsharedMaterial];
        
        // Select the material.
        self.selectedMaterial = unsharedMaterial;
        
        // Animate the material.
        CABasicAnimation *highlightAnimation = [CABasicAnimation animationWithKeyPath:@"contents"];
        highlightAnimation.toValue = [NSColor blueColor];
        highlightAnimation.fromValue = [NSColor blackColor];
        highlightAnimation.repeatCount = MAXFLOAT;
        highlightAnimation.autoreverses = YES;
        highlightAnimation.duration = 0.5;
        highlightAnimation.timingFunction = [CAMediaTimingFunction functionWithName:kCAMediaTimingFunctionEaseInEaseOut];
        
        //[self.selectedMaterial.emission addAnimation:highlightAnimation forKey:@"highlight"];
    }
}

#pragma mark - Mouse events

- (IBAction)popupMenuChanged:(id)sender
{
	NSPopUpButton *popup = (NSPopUpButton*)sender;

	NSLog(@"switch: %ld", [[popup selectedItem] tag]);
	
	switch ([[popup selectedItem] tag]) {
		case 0: // visible
			self.sphereMaterial.ambient.contents = [NSImage imageNamed:@"eso0932a"];
			break;
		case 1: // WMAP
			self.sphereMaterial.ambient.contents = [NSImage imageNamed:@"wmap_ilc_7yr_v4_200uK_RGB_sos"];
			break;
		case 2: // dark matter
			self.sphereMaterial.ambient.contents = [NSImage imageNamed:@"dark-matter"];
			break;
		case 3: // 2MASS
			self.sphereMaterial.ambient.contents = [NSImage imageNamed:@"allsky-2mass"];
			break;
		case 4: // IRAS
			self.sphereMaterial.ambient.contents = [NSImage imageNamed:@"iras-100um"];
			break;
		case 5: // WISE
			self.sphereMaterial.ambient.contents = [NSImage imageNamed:@"WISE-PIA15482"];
			break;
		case 6: // Moon
			self.sphereMaterial.ambient.contents = [NSImage imageNamed:@"moon-2048"];
			break;
		case 7: // Mars
			self.sphereMaterial.ambient.contents = [NSImage imageNamed:@"4096"];
			break;
		case 8: // Earth
			self.sphereMaterial.ambient.contents = [NSImage imageNamed:@"world.topo.bathy.200412.3x5400x2700"];
			break;
		default:
			;
	}
	self.sphere.materials = @[self.sphereMaterial];
}

#pragma mark - Mouse events

- (SCNVector3)hitToPoint:(SCNHitTestResult*)hit onSphere:(SCNSphere*)sphere
{
	CGFloat r = sphere.radius;
	CGFloat r2 = r * r; // for convenience
	
	CGFloat x = hit.localCoordinates.x;
	CGFloat y = hit.localCoordinates.y;
	CGFloat z = hit.localCoordinates.z;

	SCNVector3 p = SCNVector3Make(x, -y, z);
	CGFloat length2 = x*x + y*y; // from Pythagorian theorem
	
	p.z = sqrt(r2 - length2);
	
	return p;
}

/*
 Rotation links:
 http://stackoverflow.com/questions/4251716/iphone-mac-accelerate-framework-vector-scale-and-vector-normaliziation
 http://www.raywenderlich.com/12667/how-to-rotate-a-3d-object-using-touches-with-opengl
 http://www.tecgraf.puc-rio.br/~mgattass/fcg/material/shoemake92.pdf
 
 See: GLKQuaternion* functions (introduced OS 10.8, iOS 5)
 */
/*
 
 First attempts at rotating individual objects - this isn't even close to working, but it
 demonstrates hit detection.
 
// ----------------------------------------------------
- (void)mouseMoved:(NSEvent *)event
{
	NSPoint mouseLocation = [self convertPoint:[event locationInWindow] fromView:nil];
	NSLog(@"mouse moved (%.3f, %.3f)", mouseLocation.x, mouseLocation.y);
}


// ----------------------------------------------------
- (void)mouseDragged:(NSEvent *)event
{
	//NSLog(@"mouse dragged");
	NSPoint mouseLocation = [self convertPoint:[event locationInWindow] fromView:nil];
	NSArray *hits = [self hitTest:mouseLocation options:nil];
	
	if ([hits count] > 0) {
        SCNHitTestResult *hit = hits[0]; // Choose the nearest object hit.
		NSLog(@"Local coordinates: %0.3f, %0.3f, %0.3f", hit.localCoordinates.x, hit.localCoordinates.y, hit.localCoordinates.z);
		CGFloat x = hit.localCoordinates.x;
		CGFloat y = hit.localCoordinates.y;
		CGFloat z = hit.localCoordinates.z;
		self.sphereNode.rotation = SCNVector4Make(hit.localCoordinates.x, hit.localCoordinates.y, hit.localCoordinates.z, cos(x));
	}
	
	
//	rotation = rotation + 1;
//	self.sphereNode.rotation = SCNVector4Make(0, 1, 1, deg2rad(rotation));
	
}
*/

/*
// ----------------------------------------------------
- (void)mouseDown:(NSEvent *)event {
    // Convert the mouse location in screen coordinates to local coordinates, then perform a hit test with the local coordinates.
    NSPoint mouseLocation = [self convertPoint:[event locationInWindow] fromView:nil];
    NSArray *hits = [self hitTest:mouseLocation options:nil];
    
    // If there was a hit, select the nearest object; otherwise unselect.
    if ([hits count] > 0) {
        SCNHitTestResult *hit = hits[0]; // Choose the nearest object hit.
        [self selectNode:hit.node geometryElementIndex:hit.geometryIndex];
		
		NSLog(@"Local coordinates: %0.3f, %0.3f, %0.3f", hit.localCoordinates.x, hit.localCoordinates.y, hit.localCoordinates.z);
		NSLog(@"World coordinates: %0.3f, %0.3f, %0.3f", hit.worldCoordinates.x, hit.worldCoordinates.y, hit.worldCoordinates.z);
		
		CGPoint p = [hit textureCoordinatesWithMappingChannel:0];
		
		NSLog(@"Texture coordinates: %0.3f, %0.3f", p.x, p.y);
		
		self.textNode.position = hit.worldCoordinates;
		
		previousHitLocation = hit;
    }
    else {
        [self selectNode:nil geometryElementIndex:NSNotFound];
    }
    
    [super mouseDown:event];
}
*/

- (void)mouseDown:(NSEvent *)theEvent {
	_clickLocation = [theEvent locationInWindow];
	_clickLocation = [self convertPointFromBase:_clickLocation];
	_initialTransform = self.sphereNode.transform;
}

- (void)mouseDragged:(NSEvent *)theEvent {
	NSPoint point = [theEvent locationInWindow];
	point = [self convertPointFromBase:point];
	
    // Compute deltas
	CGFloat dx =  point.x - _clickLocation.x;
	CGFloat dy =  point.y - _clickLocation.y;
	
	// Convert to transforms, combine
	CATransform3D rotY = CATransform3DMakeRotation(dx * ROTATION_FACTOR, 0, 1, 0);
	CATransform3D rotX = CATransform3DMakeRotation(dy * ROTATION_FACTOR, -1, 0, 0);
	CATransform3D sphere_rotation = CATransform3DConcat(rotY, rotX);
	
    // Apply
    self.sphereNode.transform = CATransform3DConcat(_initialTransform, sphere_rotation);
}

@end
